//+------------------------------------------------------------------+
//|Trading manager(small test version)                   TM_Lite.mqh |
//|                                              Copyright 2015, Kos |
//|                              https://login.mql5.com/ru/users/kos |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Kos"
#property link      "https://login.mql5.com/ru/users/kos"
//+------------------------------------------------------------------+
//| defines                                                          |
//+------------------------------------------------------------------+
#define  OPERATION_SUCCEEDED          "ok.wav"
#define  OPERATION_FAILED             "timeout.wav"  
//---Execution mode consts
#define  REQUEST                      (1)
#define  INSTANT                      (2)
#define  MARKET                       (3)
#define  EXCHANGE                     (4)
//---Trade disabled reason
#define  FULL_TRADE                   (0)
#define  TERMINAL_NOT_CONNECTED       (-1)
#define  TRADE_NOT_ALLOWED            (-2)
#define  SYMBOL_TRADE_DISABLED        (-3)
#define  TRADE_MODE_LONGONLY          (-4)
#define  TRADE_MODE_SHORTONLY         (-5)
#define  TRADE_MODE_CLOSEONLY         (-6)
#define  MARKET_CLOSED                (-7)
#define  ACCOUNT_TRADE_DISABLED       (-8)
#define  ACCOUNT_EXPERT_DISABLED      (-9)
#define  INCORRECT_SYMBOL             (-10)
#define  UNKNOW                       (0)
//---Calculate mode
#define  FOREX                        (0)
#define  CFD                          (1)
#define  FUTURES                      (2)
#define  CFDINDEX                     (3)
#define  CFDLEVERAGE                  (4)
#define  EXCH_STOCKS                  (5)
#define  EXCH_FUTURES                 (6)
#define  FORTS_FUTURES                (7)
#define  EXCH                         (8)  
//---
#define  LOSS                         (0)
#define  TAKE                         (1)
//---
#define  DEFAULT_DEVIATION            (100)  //deviation
//---

#include <Trade\PositionInfo.mqh>
#include <Trade\SymbolInfo.mqh>
#include <TM\Errors.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct PositionInfo
  {
   ENUM_POSITION_TYPE Type;
   ulong             ExpertID;
   long              UID;
   datetime          TimeOpen;
   double            OpenPrice;
   double            StopLossPrice;
   double            TakeProfitPrice;
   double            Volume;
   double            ProfitAsMoney;
   double            ProfitAsPoints;
   double            Swap;
   string            comment;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CTradingManager
  {
private:
   CPositionInfo    *PosInfo;
   CSymbolInfo      *SymbInfo;
   //---
   string            m_SymbolName;
   int               m_digits;
   int               m_Spread;
   double            m_point;
   int               m_StopLevel;
   int               m_FreezeLevel;
   double            m_Bid;
   double            m_Ask;
   double            m_LotVolume;
   double            m_MaxLot;
   double            m_MinLot;
   double            m_LotStep;
   double            m_LotsLimit;
   bool              m_Positions;
   string            m_ResultMsg;
   bool              m_PlaySound;
   //---
   ENUM_POSITION_TYPE m_Type;
   double            m_OpenPrice;
   double            m_Lot;
   double            m_SLPrice;
   double            m_TPPrice;

   ENUM_POSITION_TYPE m_LastType;
   double            m_LastOpenPrice;
   double            m_LastLot;
   double            m_LastSLPrice;
   double            m_LastTPPrice;
protected:
   uchar             TradeMode();
   uchar             CalcMode();
   bool              UpdateSymbolInfo();
   double            GetIndicatorValue();
   double            CorrectLot(double f_CurrentLot,uchar f_VolumePercent,uint &p_IterationCount,double &p_rests);
   double            CorrectLot(double f_CurrentLot,uint &p_IterationCount,double &p_rests);
   bool              PositionIsFrozen(ENUM_POSITION_TYPE f_OPEN_POSITION,double f_SL,double f_TP);
   bool              StopsIsInvalid(ENUM_POSITION_TYPE f_OPEN_POSITION,double f_SL=0,double f_TP=0);
   bool              StopsIsInvalid(ENUM_ORDER_TYPE f_OPEN_ORDER,double f_OpenPrice=0,double f_SL=0,double f_TP=0,double f_StopPrice=0);
   bool              SendRequest(bool &ContinueRequest);
   bool              CheckCoincidence(ENUM_ORDER_TYPE f_OrderType,ENUM_POSITION_TYPE f_PositionType);

protected:
   MqlTradeRequest   Trade_request;
   ulong             m_deviation;
   ulong             m_magic;
   string            m_Comm;
   void              UpdatePositionInfo();
   double            GetLot(const ENUM_ORDER_TYPE f_trade_operation,const double f_price,const double f_percent);
   void              ClearStructures(void);
   string RequestErrStr(int f_MsgID)
     {
      string Result=" ";

      switch(f_MsgID)
        {
         case 0: Result="Invalid stops!"; break;
         case 1: Result="Requote!"; break;
         case 2: Result="Price changed!"; break;
         case 3: Result="No prices!"; break;
         case 5: Result="Market closed!"; break;
         case 6: Result="Invalid request!"; break;
         case 7: Result="Invalid volume!"; break;
         case 8: Result="Invalid price!"; break;
         case 9: Result="No money!"; break;
         case 10: Result="Request rejected"; break;
         default: return(Result);
        }
      return(Result);
     }

   string Message(int f_MsgID)
     {
      string Result=" ";

      switch(f_MsgID)
        {
         //---Trade Disabled reasons or trade modes descriptions         
         case 0: Result="Not connected!"; break;
         case 1: Result="AutoTrading is disabled!"; break;
         case 2: Result="Trade is disabled for the symbol!"; break;
         case 3: Result="Market closed!"; break;
         case 4: Result="Account trade disabled"; break;
         case 5: Result="Account experts disabled"; break;
         case 6: Result="Allowed only long positions"; break;
         case 7: Result="Allowed only short positions"; break;
         case 8: Result="Allowed only position close operations"; break;
         //---
         case 16: Result=""; break;
         //---PrintFormat function strings
         case 19: Result="%s -> %s"; break;
         case 20: Result="%s -> Error %d: %s"; break;
         case 21: Result="%s %s"; break;
         //---
         default: return(Result);
        }
      return(Result);
     }
public:
                     CTradingManager(void);
                    ~CTradingManager(void);
   MqlTradeResult    Trade_result;
   MqlTradeCheckResult Check_result;
   char              Initialize(CSymbolInfo  *p_ItemSymbInfo);
   uchar             Execution();
   double            GetProfit(double f_Lots,uchar f_TypePrice);
   //---
   bool              ModifyPosition(double f_SL=0,double f_TP=0,ulong f_Magic=0);
   bool              OpenPosition(ENUM_ORDER_TYPE f_Type,double f_Lot,double f_SL=0,double f_TP=0);
   void              SetOptions(ulong f_mn,ulong f_deviation,string f_Comm=" ");
   int               StopLevel() {return(m_StopLevel);}
   bool              CheckOpenPosition(ulong f_Magic=0);
   bool              CheckPositionChanged(void);
   bool              ClosePosition(uchar f_VolumePercent);
   bool              ReversePosition(bool f_WithSL=false,bool f_WithTP=false);
   bool              DeleteStopLoss(void){return(ModifyPosition(0,CurrentPosition.TakeProfitPrice,0));}
   bool              DeleteTakeProfit(void){return(ModifyPosition(CurrentPosition.StopLossPrice,0,0));}
   PositionInfo      CurrentPosition;
   CPositionInfo    *GetPositionObj(void) {return(PosInfo);}
   void              OnTick()           {}
   void              OnNewBar()         {}
   void              OnDeinit()         {}
   string            ResultMsg() {return(m_ResultMsg);}
   bool              OpenOrder(string f_Symb,ENUM_ORDER_TYPE f_Type,ENUM_ORDER_TYPE_TIME TTime,datetime f_Expiration,
                               double f_Lot,double f_OpenPrice,double f_SL=0,double f_TP=0,double f_StopPrice=0,string f_Comm="");
protected:
   bool IsStopped(const string function) //Checks forced shutdown of MQL5-program    
     {
      if(!IsStopped())
         return(false);
      //--- MQL5 program is stopped
      printf("%s: MQL5 program is stopped. Trading is disabled",function);
      Trade_result.retcode=TRADE_RETCODE_CLIENT_DISABLES_AT;
      return(true);
     }
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CTradingManager::CTradingManager(void)
  {
   SymbInfo                           =NULL;
   PosInfo                            =NULL;
   m_digits                           =0;
   m_Spread                           =0;
   m_point                            =0;
   m_StopLevel                        =0;
   m_FreezeLevel                      =0;
   m_Bid                              =0;
   m_Ask                              =0;
   m_deviation                        =100;
   m_magic                            =0;
   m_Comm                             ="";
   m_LotVolume                        =0;
   m_MaxLot                           =0;
   m_MinLot                           =0;
   m_LotStep                          =0;
   m_LotsLimit                        =0;
   m_Positions                        =false;
   m_ResultMsg                        =" ";
   m_SymbolName                       =Symbol();
   m_PlaySound                        =false;
//---
   m_Type=m_LastType=POSITION_TYPE_SELL;
   m_OpenPrice=m_LastOpenPrice=0.0;
   m_Lot=m_LastLot=0.0;
   m_SLPrice=m_LastSLPrice            =0.0;
   m_TPPrice=m_LastTPPrice            =0.0;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CTradingManager::~CTradingManager(void)
  {
   if(PosInfo!=NULL) delete PosInfo;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
char CTradingManager::Initialize(CSymbolInfo  *p_ItemSymbInfo)
  {
   SymbInfo=p_ItemSymbInfo;
//---
   if((PosInfo=new CPositionInfo)==NULL)
     {
      PrintFormat("%s %s",__FUNCTION__,"PositionInfo object create failed");
      return(INIT_FAILED);
     }
//---
   if(SymbInfo!=NULL && SymbInfo.Select())
     {
      m_SymbolName  =SymbInfo.Name();
      m_digits      =SymbInfo.Digits();
      m_StopLevel   =SymbInfo.StopsLevel();
      m_FreezeLevel =SymbInfo.FreezeLevel();
      SymbInfo.InfoDouble(SYMBOL_POINT,m_point);
      SymbInfo.InfoDouble(SYMBOL_VOLUME_MAX,m_MaxLot);
      SymbInfo.InfoDouble(SYMBOL_VOLUME_MIN,m_MinLot);
      SymbInfo.InfoDouble(SYMBOL_VOLUME_STEP,m_LotStep);
      UpdateSymbolInfo();
     }
   else return(INIT_FAILED);
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CTradingManager::UpdateSymbolInfo()
  {
   if(SymbInfo.RefreshRates())
     {
      m_Bid      =SymbInfo.Bid();
      m_Ask      =SymbInfo.Ask();
      m_Spread   =SymbInfo.Spread();
      return(true);
     }
   return(NULL);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CTradingManager::UpdatePositionInfo()
  {
   SymbInfo.RefreshRates();
   uchar _MarketType=FOREX;
   uchar _CalcMode=CalcMode();
   double _last=0.0;
   UpdateSymbolInfo();
   switch(_CalcMode)
     {
      case FOREX:
      case CFD:
      case CFDINDEX:
      case CFDLEVERAGE:
        {
         _MarketType=FOREX;
        }
      break;

      case EXCH_STOCKS:
      case EXCH_FUTURES:
      case FORTS_FUTURES:
      case FUTURES:
        {
         _last=(double)NormalizeDouble(SymbInfo.Last(),m_digits);
         _MarketType=EXCH;
        }
      break;
     }

   int _pips=(m_digits>0)?(int)MathPow(10,m_digits):1;

   if(PosInfo.Select(m_SymbolName))
     {
      CurrentPosition.Type            =PosInfo.PositionType();
      CurrentPosition.ExpertID        =PosInfo.Magic();
      CurrentPosition.UID             =PosInfo.Identifier();
      CurrentPosition.TimeOpen        =PosInfo.Time();
      CurrentPosition.OpenPrice       =PosInfo.PriceOpen();
      CurrentPosition.StopLossPrice   =PosInfo.StopLoss();
      CurrentPosition.TakeProfitPrice =PosInfo.TakeProfit();
      CurrentPosition.Volume          =PosInfo.Volume();
      CurrentPosition.ProfitAsMoney   =PosInfo.Profit();
      CurrentPosition.comment         =PosInfo.Comment();
      if(_MarketType==FOREX)
         CurrentPosition.ProfitAsPoints=(CurrentPosition.Type==POSITION_TYPE_SELL)?(double)((CurrentPosition.OpenPrice-m_Ask)*_pips):(double)((m_Bid-CurrentPosition.OpenPrice)*_pips);
      else
         CurrentPosition.ProfitAsPoints=(CurrentPosition.Type==POSITION_TYPE_SELL)?(double)((CurrentPosition.OpenPrice-_last)*_pips):(double)((_last-CurrentPosition.OpenPrice)*_pips);
      CurrentPosition.Swap=PosInfo.Swap();
     }
  }
//+------------------------------------------------------------------+
//|Execution mode                                                    |
//+------------------------------------------------------------------+
uchar CTradingManager::Execution()
  {
   switch(SymbInfo.TradeExecution())
     {
      case SYMBOL_TRADE_EXECUTION_REQUEST:   return(REQUEST);
      case SYMBOL_TRADE_EXECUTION_INSTANT:   return(INSTANT);
      case SYMBOL_TRADE_EXECUTION_MARKET:    return(MARKET);
      case SYMBOL_TRADE_EXECUTION_EXCHANGE:  return(EXCHANGE);
     }
   return(UCHAR_MAX);
  }
//+------------------------------------------------------------------+
//| Trading mode                                                     |
//+------------------------------------------------------------------+
uchar CTradingManager::TradeMode()
  {
   switch(SymbInfo.TradeMode())
     {
      case SYMBOL_TRADE_MODE_DISABLED:     return(SYMBOL_TRADE_DISABLED);
      case SYMBOL_TRADE_MODE_LONGONLY:     return(TRADE_MODE_LONGONLY);
      case SYMBOL_TRADE_MODE_SHORTONLY:    return(TRADE_MODE_SHORTONLY);
      case SYMBOL_TRADE_MODE_CLOSEONLY:    return(TRADE_MODE_CLOSEONLY);
      case SYMBOL_TRADE_MODE_FULL:         return(FULL_TRADE);
      default:                             return(UNKNOW);
     }
  }
//+------------------------------------------------------------------+
//|The method of calculating the margin                              |
//+------------------------------------------------------------------+
uchar CTradingManager::CalcMode()
  {
   switch(SymbInfo.TradeCalcMode())
     {
      case SYMBOL_CALC_MODE_FOREX:               return(FOREX);
      case SYMBOL_CALC_MODE_CFD:                 return(CFD);
      case SYMBOL_CALC_MODE_FUTURES:             return(FUTURES);
      case SYMBOL_CALC_MODE_CFDINDEX:            return(CFDINDEX);
      case SYMBOL_CALC_MODE_CFDLEVERAGE:         return(CFDLEVERAGE);
      case SYMBOL_CALC_MODE_EXCH_STOCKS:         return(EXCH_STOCKS);
      case SYMBOL_CALC_MODE_EXCH_FUTURES:        return(EXCH_FUTURES);
      case SYMBOL_CALC_MODE_EXCH_FUTURES_FORTS:  return(FORTS_FUTURES);
      default: return(UCHAR_MAX);
     }
   return(UCHAR_MAX);
  }
//+------------------------------------------------------------------+
//| Access functions OrderCalcMargin(...).                           |
//| INPUT:  trade_operation - f_trade operation,                     |
//|         price           - f_price of the opening position,       |
//|         percent         - f_percent of available margin [1-100%].|
//+------------------------------------------------------------------+
double CTradingManager::GetLot(const ENUM_ORDER_TYPE f_trade_operation,const double f_price,const double f_percent)
  {
   UpdateSymbolInfo();
   uchar  _CalcMode=CalcMode();
   double _MarginInitial=0.0;
   double _Percent=f_percent/100.0;
   double _FreeMargin=AccountInfoDouble(ACCOUNT_FREEMARGIN);
   double _ContractSize=SymbInfo.ContractSize();

//--- checks
   if(m_SymbolName=="" || f_price<=0.0)
     {
      Print(__FUNCTION__," - invalid parameters!");
      return(0.0);
     }
   if(_FreeMargin<=0) return(-1);

//--- calculate margin requirements for 1 lot
   switch(_CalcMode)
     {
      case FOREX:
        {
         ResetLastError();
         if(!OrderCalcMargin(f_trade_operation,m_SymbolName,1.0,f_price,_MarginInitial) || _MarginInitial<0.0)
           {
            Print(__FUNCTION__," - margin calculation failed! ",GetErrorDesc(GetLastError()));
            return(0.0);
           }
        }
      break;

      case CFD:
      case CFDINDEX:
      case CFDLEVERAGE:
        {
         ResetLastError();
         if(!OrderCalcMargin(f_trade_operation,m_SymbolName,1.0,f_price,_MarginInitial) || _MarginInitial<0.0)
           {
            Print(__FUNCTION__," - margin calculation failed! ",GetErrorDesc(GetLastError()));
            return(0.0);
           }
        }
      break;

      case EXCH_STOCKS:
        {
        }
      break;

      case EXCH_FUTURES:
        {
         _MarginInitial=SymbInfo.MarginInitial();
        }
      break;

      case FORTS_FUTURES:
        {
         _MarginInitial=SymbInfo.MarginInitial();
        }
      break;

      default: break;
     }

//---
   if(_MarginInitial==0.0)
     {
      return(SymbInfo.LotsMin());
     }
//--- calculate maximum volume
   double _volume=NormalizeDouble(_FreeMargin*_Percent/_MarginInitial,2);

//--- normalize and check limits
   if(m_LotStep>0.0) _volume=m_LotStep*MathFloor(_volume/m_LotStep);
   if(_volume<m_MinLot) return(0.0);
   if(_volume>m_MaxLot) return(m_MaxLot);
   return(_volume);
  }
//+------------------------------------------------------------------+
//|Clear structures                                                  |
//+------------------------------------------------------------------+
void CTradingManager::ClearStructures(void)
  {
   ZeroMemory(Trade_request);
   ZeroMemory(Trade_result);
   ZeroMemory(Check_result);
  }
//+------------------------------------------------------------------+
//|Checking the minimum distance StopsLevel when modifying StopLoss  |
//| and/or TakeProfit orders                                         |
//| Parameters:                                                      |
//|  -position type                                                  |
//|  -new price StopLoss                                             |
//|  -new price TakeProfit                                           |
//+------------------------------------------------------------------+  
bool  CTradingManager::StopsIsInvalid(ENUM_POSITION_TYPE f_OPEN_POSITION,double f_SL=0,double f_TP=0)
  {
   bool IsInvalid=NULL;
   UpdatePositionInfo();
   int _stopLvl=(m_StopLevel!=0)?(ushort)m_StopLevel:m_Spread;
   switch(f_OPEN_POSITION)
     {
      case POSITION_TYPE_BUY:
         IsInvalid=((f_SL!=0 && m_Bid!=0 && NormalizeDouble(m_Bid-f_SL,m_digits)/m_point<_stopLvl) || 
                    (f_TP!=0 && m_Bid!=0 && NormalizeDouble(f_TP-m_Bid,m_digits)/m_point<_stopLvl));
      break;

      case POSITION_TYPE_SELL:
         IsInvalid=((f_SL!=0 && m_Ask!=0 && NormalizeDouble(f_SL-m_Ask,m_digits)/m_point<_stopLvl) || 
                    (f_TP!=0 && m_Ask!=0 && NormalizeDouble(m_Ask-f_TP,m_digits)/m_point<_stopLvl));
      break;
     }
   return(IsInvalid);
  }
//+------------------------------------------------------------------+
//|Check the minimum distance StopsLevel when opening a position and |
//| setups pending orders                                            |
//|Parameters:                                                       |
//|  - order type                                                    |
//|  - open price                                                    |
//|  - StopLoss price                                                |
//|  - TakeProfit price                                              |
//|  - price when reaching which will be exhibited pending orders    |
//|    StopLimit or BuyLimit for the price: OpenPrice(StopLimit)     |
//+------------------------------------------------------------------+  
bool  CTradingManager::StopsIsInvalid(ENUM_ORDER_TYPE f_OPEN_ORDER,double f_OpenPrice=0,double f_SL=0,double f_TP=0,double f_StopPrice=0)
  {
   bool IsInvalid=NULL;
   UpdateSymbolInfo();
   int _stopLvl=(m_StopLevel!=0)?(ushort)m_StopLevel:m_Spread;
//---
   switch(f_OPEN_ORDER)
     {
      case ORDER_TYPE_BUY:
        {
         IsInvalid=((f_SL!=0 && m_Bid!=0 && NormalizeDouble(m_Bid-f_SL,m_digits)/m_point<_stopLvl) || 
                    (f_TP!=0 && m_Ask!=0 && NormalizeDouble(f_TP-m_Ask,m_digits)/m_point<_stopLvl));
        }
      break;

      case ORDER_TYPE_BUY_LIMIT:
        {
         IsInvalid=((m_Ask!=0 && NormalizeDouble(m_Ask-f_OpenPrice,m_digits)/m_point<_stopLvl) || 
                    (f_SL!=0 && NormalizeDouble(f_OpenPrice-f_SL,m_digits)/m_point<_stopLvl) ||
                    (f_TP!=0 && NormalizeDouble(f_TP-f_OpenPrice,m_digits)/m_point<_stopLvl));
        }
      break;

      case ORDER_TYPE_BUY_STOP:
        {
         IsInvalid=((m_Ask!=0 && NormalizeDouble(f_OpenPrice-m_Ask,m_digits)/m_point<_stopLvl) || 
                    (f_SL!=0 && NormalizeDouble(f_OpenPrice-f_SL,m_digits)/m_point<_stopLvl) ||
                    (f_TP!=0 && NormalizeDouble(f_TP-f_OpenPrice,m_digits)/m_point<_stopLvl));
        }
      break;

      case ORDER_TYPE_BUY_STOP_LIMIT:
        {
         IsInvalid=((m_Ask!=0 && NormalizeDouble(f_StopPrice-m_Ask,m_digits)/m_point<_stopLvl) || 
                    NormalizeDouble(f_StopPrice-f_OpenPrice,m_digits)/m_point<_stopLvl || 
                    (f_SL!=0 && NormalizeDouble(f_OpenPrice-f_SL,m_digits)/m_point<_stopLvl) ||
                    (f_TP!=0 && NormalizeDouble(f_TP-f_OpenPrice,m_digits)/m_point<_stopLvl));
        }
      break;

      case ORDER_TYPE_SELL:
        {
         IsInvalid=((f_SL!=0 && m_Ask!=0 && NormalizeDouble(f_SL-m_Ask,m_digits)/m_point<_stopLvl) || 
                    (f_TP!=0 && m_Bid!=0 && NormalizeDouble(m_Bid-f_TP,m_digits)/m_point<_stopLvl));
        }
      break;

      case ORDER_TYPE_SELL_LIMIT:
        {
         IsInvalid=((m_Bid!=0 && NormalizeDouble(f_OpenPrice-m_Bid,m_digits)/m_point<_stopLvl) || 
                    (f_SL!=0 && NormalizeDouble(f_SL-f_OpenPrice,m_digits)/m_point<_stopLvl) ||
                    (f_TP!=0 && NormalizeDouble(f_OpenPrice-f_TP,m_digits)/m_point<_stopLvl));
        }
      break;

      case ORDER_TYPE_SELL_STOP:
        {
         IsInvalid=((m_Bid!=0 && NormalizeDouble(m_Bid-f_OpenPrice,m_digits)/m_point<_stopLvl) || 
                    (f_SL!=0 && NormalizeDouble(f_SL-f_OpenPrice,m_digits)/m_point<_stopLvl) ||
                    (f_TP!=0 && NormalizeDouble(f_OpenPrice-f_TP,m_digits)/m_point<_stopLvl));
        }
      break;

      case ORDER_TYPE_SELL_STOP_LIMIT:
        {
         IsInvalid=((m_Bid!=0 && NormalizeDouble(m_Bid-f_StopPrice,m_digits)/m_point<_stopLvl) || 
                    NormalizeDouble(f_OpenPrice-f_StopPrice,m_digits)/m_point<_stopLvl || 
                    (f_SL!=0 && NormalizeDouble(f_SL-f_OpenPrice,m_digits)/m_point<_stopLvl) ||
                    (f_TP!=0 && NormalizeDouble(f_OpenPrice-f_TP,m_digits)/m_point<_stopLvl));
        }
      break;
     }
   return(IsInvalid);
  }
//+------------------------------------------------------------------+
//|Check freeze-distance open positions                              |
//+------------------------------------------------------------------+
bool CTradingManager::PositionIsFrozen(ENUM_POSITION_TYPE f_OPEN_POSITION,double f_SL,double f_TP)
  {
   bool Frozen=NULL;
   UpdateSymbolInfo();
   switch(f_OPEN_POSITION)
     {
      case POSITION_TYPE_BUY:
         Frozen=((f_TP!=0 && m_Bid!=0 && NormalizeDouble(f_TP-m_Bid,m_digits)/m_point<m_FreezeLevel) || 
                 (f_SL!=0 && m_Bid!=0 && NormalizeDouble(m_Bid-f_SL,m_digits)/m_point<m_FreezeLevel));
      break;

      case POSITION_TYPE_SELL:
         Frozen=((f_TP!=0 && m_Ask!=0 && NormalizeDouble(m_Ask-f_TP,m_digits)/m_point<m_FreezeLevel) || 
                 (f_SL!=0 && m_Ask!=0 && NormalizeDouble(f_SL-m_Ask,m_digits)/m_point<m_FreezeLevel));
      break;
     }
   return(Frozen);
  }
//+------------------------------------------------------------------+
//|Returns the value possible profit for the specified trading volume|
//+------------------------------------------------------------------+
double CTradingManager::GetProfit(double f_Lots,uchar f_TypePrice)
  {
   double result=0.0;
   ENUM_POSITION_TYPE _Type=POSITION_TYPE_BUY;
   ENUM_ORDER_TYPE _OrderType=ORDER_TYPE_BUY;
   double _OP=0.0;
   double _SL=0.0;
   double _TP=0.0;
   if(CheckOpenPosition())
     {
      _OP=CurrentPosition.OpenPrice;
      _OrderType=(CurrentPosition.Type==POSITION_TYPE_BUY)?ORDER_TYPE_BUY:ORDER_TYPE_SELL;
      switch(f_TypePrice)
        {
         case LOSS:
           {
            _SL=CurrentPosition.StopLossPrice;
            if(_SL!=0 && _OP!=0)
              {
               if(!OrderCalcProfit(_OrderType,m_SymbolName,f_Lots,_OP,_SL,result)) Print("Profit calculate failed");
              }
           }
         break;

         case TAKE:
           {
            _TP=CurrentPosition.TakeProfitPrice;
            if(_TP!=0 && _OP!=0)
              {
               if(!OrderCalcProfit(_OrderType,m_SymbolName,f_Lots,_OP,_TP,result)) Print("Profit calculate failed");
              }
           }
         break;
        }

     }
   return(result);
  }
//+------------------------------------------------------------------+
//|Setup parameters: ExpertID,Deviation,Comment                      |
//+------------------------------------------------------------------+
void  CTradingManager::SetOptions(ulong f_mn,
                                  ulong f_deviation,
                                  string  f_Comm=" ")
  {
   m_magic=f_mn;
   m_deviation=(f_deviation>=DEFAULT_DEVIATION)?f_deviation:DEFAULT_DEVIATION;
   m_Comm=f_Comm;
  }
//+------------------------------------------------------------------+
//| Checking the open position                                       |
//+------------------------------------------------------------------+
bool CTradingManager::CheckOpenPosition(ulong f_Magic=0)
  {
   if(PosInfo.Select(m_SymbolName))
     {
      UpdatePositionInfo();
      if(CurrentPosition.ExpertID==f_Magic || f_Magic==0) return(m_Positions=true);
     }
   return(m_Positions=false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CTradingManager::CheckPositionChanged(void)
  {
   m_Type     =CurrentPosition.Type;
   m_OpenPrice=NormalizeDouble(CurrentPosition.OpenPrice,m_digits);
   m_Lot      =CurrentPosition.Volume;
   m_TPPrice  =NormalizeDouble(CurrentPosition.TakeProfitPrice,m_digits);
   m_SLPrice  =NormalizeDouble(CurrentPosition.StopLossPrice,m_digits);

   if(m_Type     !=m_LastType      ||
      m_OpenPrice!=m_LastOpenPrice ||
      m_TPPrice  !=m_LastTPPrice   ||
      m_SLPrice  !=m_LastSLPrice   ||
      m_Lot      !=m_LastLot)
     {
      m_LastType      = m_Type;
      m_LastOpenPrice = m_OpenPrice;
      m_LastTPPrice   = m_TPPrice;
      m_LastSLPrice   = m_SLPrice;
      m_LastLot       = m_Lot;
      return(true);
     }
   return(false);
  }
//+------------------------------------------------------------------+
//|Modify position                                                   |
//|*before you call this function,you must call the function:        |
//| CheckOpenPositions(Magic)                                        |
//+------------------------------------------------------------------+
bool CTradingManager::ModifyPosition(double f_SL=0,double f_TP=0,ulong f_Magic=0)
  {
   bool Done=false;

   if(IsStopped(__FUNCTION__))   return(false);

   if(CurrentPosition.ExpertID==f_Magic || f_Magic==0)
     {
      ClearStructures();
      //--- checking stopslevel
      if(!StopsIsInvalid(CurrentPosition.Type,f_SL,f_TP))
        {
         do
           {
            bool Continue=NULL;
            Trade_request.action    =TRADE_ACTION_SLTP;
            Trade_request.symbol    =m_SymbolName;
            Trade_request.sl        =f_SL;
            Trade_request.tp        =f_TP;
            Trade_request.deviation =m_deviation;

            if((Done=SendRequest(Continue))) UpdatePositionInfo();
            else
              {
               if(Continue) Done=false;
               else return(false);
              }
           }
         while(!Done);
        }
      else
        {
         PlaySound(OPERATION_FAILED);
         m_ResultMsg=RequestErrStr(0);
        }
     }
   return(Done);
  }
//+------------------------------------------------------------------+
//|Check the coincidence of the direction of trade                   |
//+------------------------------------------------------------------+
bool CTradingManager::CheckCoincidence(ENUM_ORDER_TYPE f_OrderType,ENUM_POSITION_TYPE f_PositionType)
  {
   bool coincide=NULL;
   coincide=((f_PositionType==POSITION_TYPE_SELL && (f_OrderType==ORDER_TYPE_SELL || f_OrderType==ORDER_TYPE_SELL_LIMIT || f_OrderType==ORDER_TYPE_SELL_STOP || f_OrderType==ORDER_TYPE_SELL_STOP_LIMIT)) || 
             (f_PositionType==POSITION_TYPE_BUY && (f_OrderType==ORDER_TYPE_BUY || f_OrderType==ORDER_TYPE_BUY_LIMIT || f_OrderType==ORDER_TYPE_BUY_STOP || f_OrderType==ORDER_TYPE_BUY_STOP_LIMIT)));
   return(coincide);
  }
//+------------------------------------------------------------------+
//|Correction of the volume required for opening position            |
//+------------------------------------------------------------------+

double  CTradingManager::CorrectLot(double f_CurrentLot,uint &p_IterationCount,double &p_rests)
  {
   if(f_CurrentLot<=m_MaxLot) return(f_CurrentLot);
   p_rests=f_CurrentLot-m_MaxLot;
   p_IterationCount=(int)MathCeil(f_CurrentLot/m_MaxLot);
   Print("Position on "+m_SymbolName+" will be opened by "+IntegerToString(p_IterationCount)+" parts.");
   return(m_MaxLot);
  }
//+------------------------------------------------------------------+
//|Open position                                                     |
//+------------------------------------------------------------------+
bool CTradingManager::OpenPosition(ENUM_ORDER_TYPE f_Type,double f_Lot,double f_SL=0,double f_TP=0)
  {
   uint attempts=0;
   bool   Done=false;
   double Volume=f_Lot;
   uint filling=(uint)SymbolInfoInteger(m_SymbolName,SYMBOL_FILLING_MODE);
   
   //---
   if(!AccountInfoInteger(ACCOUNT_TRADE_ALLOWED)) {PrintFormat(Message(19),m_SymbolName,Message(4)); return(Done);}
   if(!AccountInfoInteger(ACCOUNT_TRADE_EXPERT)) {PrintFormat(Message(19),m_SymbolName,Message(5)); return(Done);}
   //---
    
   if(IsStopped(__FUNCTION__))   return(Done);
   
   UpdateSymbolInfo();
   ClearStructures();

   if(f_Type!=ORDER_TYPE_BUY && f_Type!=ORDER_TYPE_SELL) return(Done);
   switch(TradeMode()) //--  
     {
      case SYMBOL_TRADE_DISABLED:
         PrintFormat(Message(19),m_SymbolName,Message(2));
         return(Done);

      case TRADE_MODE_CLOSEONLY:
        {
         PrintFormat(Message(19),m_SymbolName,Message(8));
         if(m_Positions) //    
           {
            if(CheckCoincidence(f_Type,CurrentPosition.Type)) return(Done);              //      ,    
            else                                                                         //   
              {
               Volume=(CurrentPosition.Volume<Volume) ? CurrentPosition.Volume : Volume;  //     ,      
              }
           }
         else return(Done);
        }
      break;

      case TRADE_MODE_LONGONLY:
        {
         if(f_Type!=ORDER_TYPE_BUY)
           {
            PrintFormat(Message(19),m_SymbolName,Message(6));
            return(Done);
           }
        }
      break;

      case TRADE_MODE_SHORTONLY:
        {
         if(f_Type!=ORDER_TYPE_SELL)
           {
            PrintFormat(Message(19),m_SymbolName,Message(7));
            return(Done);
           }
        }
      break;

      default: break;
     }
   
   bool   NeedModify=NULL;
//--- checking stopslevel
   if(!StopsIsInvalid(f_Type,0,f_SL,f_TP))
     {
      double CurrentLot=Volume;
      double rests=0.0;
      double VolumeForOpen=0.0;
      int    IterationCount=0;
      uchar  EM=Execution();  //--Execution mode

      do
        {
         double OpenPrice=(f_Type==ORDER_TYPE_BUY) ? m_Ask : m_Bid;
         switch(EM)
           {
            case MARKET:
              {
               NeedModify=(f_SL!=0 || f_TP!=0);
               Trade_request.sl       =f_SL;
               Trade_request.tp       =f_TP;
              }
            break;

            case EXCHANGE:
            case REQUEST:
            case INSTANT:
              {
               NeedModify=CheckCoincidence(f_Type,CurrentPosition.Type);
               Trade_request.price    =OpenPrice;
               Trade_request.sl       =f_SL;
               Trade_request.tp       =f_TP;
              }
            break;

            case UNKNOW:  return(Done);
           }

         bool Continue=NULL;
         //--- setting request
         Trade_request.action         =TRADE_ACTION_DEAL;
         Trade_request.type           =f_Type;
         Trade_request.symbol         =m_SymbolName;
         Trade_request.volume         =VolumeForOpen=(rests==0)?CorrectLot(CurrentLot,IterationCount,rests):CorrectLot(rests,IterationCount,rests);
         //Trade_request.type_filling   =(EM==EXCHANGE)?ORDER_FILLING_IOC:ORDER_FILLING_FOK;
         if((filling  &SYMBOL_FILLING_FOK)!=0) Trade_request.type_filling=ORDER_FILLING_FOK;
         else
           {
            if((filling  &SYMBOL_FILLING_IOC)!=0) Trade_request.type_filling=ORDER_FILLING_IOC;
             else Trade_request.type_filling=ORDER_FILLING_RETURN;
           }
       
         Trade_request.deviation      =m_deviation;
         Trade_request.magic          =m_magic;
         Trade_request.comment        =m_Comm;
         //---
         Done=SendRequest(Continue);
         if(Done)
           {
            if(IterationCount>0) IterationCount--;
            if(!(Done=(IterationCount==0))) Sleep(500);
           }
         else
           {
            if(Continue)
              {
               if(Trade_result.retcode==TRADE_RETCODE_REQUOTE ||
                  Trade_result.retcode==TRADE_RETCODE_PRICE_CHANGED ||
                  Trade_result.retcode==TRADE_RETCODE_REJECT)
                 {
                  attempts++;
                 }
               else rests+=VolumeForOpen;
              }
            else return(false);
           }
        }
      while(!Done && attempts<5);
     }
   else
     {
      PlaySound(OPERATION_FAILED);
      m_ResultMsg=RequestErrStr(0);
     }

   if(Done && NeedModify)
     {
      if(PosInfo.Select(m_SymbolName)) ModifyPosition(f_SL,f_TP,m_magic);
     }
   return(Done);
  }
//+------------------------------------------------------------------+
//|Correction of the volume needed for closing                       |
//+------------------------------------------------------------------+

double  CTradingManager::CorrectLot(double f_CurrentLot,uchar f_VolumePercent,uint &p_IterationCount,double &p_rests)
  {
   if(f_CurrentLot==m_MinLot) return(f_CurrentLot);

   double k=NULL;
   double NeedVolume=NULL;
   double part=NULL;
   uchar  Percent=f_VolumePercent;

   if(Percent==0 || Percent>100) Percent=100;      //fully close
   if(Percent!=100)
     {
      if(m_MinLot==0) m_MinLot=0.1;
      if(m_MaxLot==0) m_MaxLot=100;
      if(m_LotStep>0) k=1/m_LotStep; else k=1/m_MinLot;
      part=(f_CurrentLot*Percent)/100;
      NeedVolume=MathFloor(part*k)/k;
      if(NeedVolume<=m_MinLot || f_CurrentLot-NeedVolume<=m_MinLot) return(f_CurrentLot);
     }
   else NeedVolume=f_CurrentLot;
   if(NeedVolume>m_MaxLot)
     {
      p_rests=NeedVolume-m_MaxLot;
      p_IterationCount=(int)MathCeil(NeedVolume/m_MaxLot);
      Print("Position on "+m_SymbolName+" will be closed by "+IntegerToString(p_IterationCount)+" parts.");
      return(m_MaxLot);
     }
   return(NeedVolume);
  }
//+------------------------------------------------------------------+
//|Close position                                                    |
//|*before you call this function,you must call the function:        |
//| CheckOpenPositions(Magic)                                        |
//+------------------------------------------------------------------+

bool CTradingManager::ClosePosition(uchar f_VolumePercent)
  {

   bool   Done=false;
   uint   IterationCount=0;

   if(IsStopped(__FUNCTION__))   return(false);

   if(!PositionIsFrozen(PosInfo.PositionType(),PosInfo.StopLoss(),PosInfo.TakeProfit()))
     {
      double VolumeForClose=0.0;
      double CurrentVolume=CurrentPosition.Volume;
      double rests=0.0;                                                    //  ,      SYMBOL_VOLUME_MAX
      bool   Continue=false;
      do
        {
         VolumeForClose=(rests==0) ? CorrectLot(CurrentVolume,f_VolumePercent,IterationCount,rests) : CorrectLot(rests,100,IterationCount,rests);
         NormalizeDouble(VolumeForClose,2);
         if(CurrentPosition.Type==POSITION_TYPE_BUY)
           {
            Done=OpenPosition(ORDER_TYPE_SELL,VolumeForClose,0,0);
           }
         else
           {
            Done=OpenPosition(ORDER_TYPE_BUY,VolumeForClose,0,0);
           }

         if(Done)
           {
            if(IterationCount>0)
              {
               IterationCount--;
               if(!(Done=(IterationCount==0))) Sleep(500);
              }
            else    return(true);
           }
         else   return(false);
        }
      while(!Done);
     }
   return(false);
  }
//+------------------------------------------------------------------+
//|Reverse position(Buy -> Sell or Sell -> Buy)                      |
//|*before you call this function,you must call the function:        |
//| CheckOpenPositions(Magic)                                        |
//+------------------------------------------------------------------+
bool CTradingManager::ReversePosition(bool f_WithSL=false,bool f_WithTP=false)
  {
   bool result=true;
//---current position parameters
   ENUM_POSITION_TYPE _Type=CurrentPosition.Type;
   double _OpenPrice=(f_WithSL || f_WithTP)?(double)NormalizeDouble(CurrentPosition.OpenPrice,m_digits):0.0;
   double _volume=CurrentPosition.Volume;
   double _SL_Price = (f_WithSL)?(double)NormalizeDouble(CurrentPosition.StopLossPrice,m_digits):0.0;
   double _TP_Price = (f_WithTP)?(double)NormalizeDouble(CurrentPosition.TakeProfitPrice,m_digits):0.0;
   double _SL_Points = 0.0;
   double _TP_Points = 0.0;
   m_Comm=CurrentPosition.comment;
   m_magic=CurrentPosition.ExpertID;

   if(m_point!=0)
     {
      if(_OpenPrice>0 && _SL_Price>0) _SL_Points = (_Type==POSITION_TYPE_SELL)?NormalizeDouble(((_SL_Price-_OpenPrice)/m_point),0):NormalizeDouble(((_OpenPrice-_SL_Price)/m_point),0);
      if(_OpenPrice>0 && _TP_Price>0) _TP_Points = (_Type==POSITION_TYPE_SELL)?NormalizeDouble(((_OpenPrice-_TP_Price)/m_point),0):NormalizeDouble(((_TP_Price-_OpenPrice)/m_point),0);
     }

//---New position parameters
   ENUM_ORDER_TYPE Type=(_Type==POSITION_TYPE_SELL)?ORDER_TYPE_BUY:ORDER_TYPE_SELL;
   ENUM_POSITION_TYPE c_Type;
   double volume=2*_volume;
   double TP_Price=0.0;
   double SL_Price=0.0;
   double OP_Price=0.0;
   bool NeedModify=(_SL_Points>0 || _TP_Points>0);
   result&=OpenPosition(Type,volume);

   if(!result)
     {
      result&=ClosePosition(100);
      result&=OpenPosition(Type,_volume);
     }
   else
     {
      if(NeedModify)
        {
         Sleep(500);
         if(CheckOpenPosition())
           {
            c_Type=CurrentPosition.Type;
            OP_Price=CurrentPosition.OpenPrice;
            if(_TP_Points>0) TP_Price=(c_Type==POSITION_TYPE_SELL)?OP_Price-NormalizeDouble(_TP_Points*m_point,m_digits):OP_Price+NormalizeDouble(_TP_Points*m_point,m_digits);
            if(_SL_Points>0) SL_Price=(c_Type==POSITION_TYPE_SELL)?OP_Price+NormalizeDouble(_SL_Points*m_point,m_digits):OP_Price-NormalizeDouble(_SL_Points*m_point,m_digits);
            result&=ModifyPosition(SL_Price,TP_Price,m_magic);
           }
        }
     }
   return(result);
  }
//+------------------------------------------------------------------+
//|Set pending orders                                                |
//+------------------------------------------------------------------+

bool CTradingManager::OpenOrder(string f_Symb,
                                ENUM_ORDER_TYPE f_Type,
                                ENUM_ORDER_TYPE_TIME TTime,
                                datetime f_Expiration,
                                double f_Lot,
                                double f_OpenPrice,//    ,   ...STOP_LIMIT   Limit ,
                                double f_SL=0,
                                double f_TP=0,
                                double f_StopPrice=0,//       ...Limit 
                                string f_Comm="")
  {
   bool Done=NULL;
   if(f_Type==ORDER_TYPE_BUY && f_Type==ORDER_TYPE_SELL) return(Done);

   double Volume=f_Lot;
   switch(TradeMode()) //-- trading mode
     {
      case SYMBOL_TRADE_DISABLED:
        {
         Print(Message(2));
         return(Done);
        }
      break;

      case TRADE_MODE_CLOSEONLY:
        {
         Print(Message(8));
         if(m_Positions) //    
           {
            if(CheckCoincidence(f_Type,CurrentPosition.Type)) return(Done);               //        ,    
            else                                                                         //   
              {
               Volume=(CurrentPosition.Volume<Volume) ? CurrentPosition.Volume : Volume;  //     ,      
              }
           }
         else return(Done);
        }
      break;

      case TRADE_MODE_LONGONLY:
        {
         if(f_Type!=ORDER_TYPE_BUY)
           {
            Print(Message(6));
            return(Done);
           }
        }
      break;

      case TRADE_MODE_SHORTONLY:
        {
         if(f_Type!=ORDER_TYPE_SELL)
           {
            Print(Message(7));
            return(Done);
           }
        }
      break;

      default: break;
     }
//--- checking stopslevel
   if(!StopsIsInvalid(f_Type,f_OpenPrice,f_SL,f_TP,f_StopPrice))
     {
      ClearStructures();
      do
        {
         bool Continue=NULL;
         //--- setting request
         Trade_request.action       =TRADE_ACTION_PENDING;
         Trade_request.type         =f_Type;
         Trade_request.symbol       =m_SymbolName;
         Trade_request.volume       =Volume;
         Trade_request.price        =f_OpenPrice;
         Trade_request.stoplimit    =f_StopPrice;
         Trade_request.sl           =f_SL;
         Trade_request.tp           =f_TP;
         Trade_request.type_filling =ORDER_FILLING_FOK;
         Trade_request.magic        =m_magic;
         Trade_request.type_time    =TTime;
         Trade_request.expiration   =f_Expiration;
         Trade_request.comment      =f_Comm;
         //---
         if(!(Done=SendRequest(Continue)))
           {
            if(!Continue) return(false);
           }
        }
      while(!Done);
     }
   else m_ResultMsg=RequestErrStr(0);
   return(Done);
  }
//+------------------------------------------------------------------+
//|Sending trade request                                             |
//+------------------------------------------------------------------+

bool CTradingManager::SendRequest(bool &ContinueRequest)
  {
   uint   CheckRetCode=NULL;
   bool   Done        =false;
   bool   Result      =false;

   if(OrderCheck(Trade_request,Check_result)) // checking the correctness of the structure of trade request
     {
      Result=OrderSend(Trade_request,Trade_result);
      switch(Trade_result.retcode)
        {
         //---Request done
         case TRADE_RETCODE_PLACED:
         case TRADE_RETCODE_DONE:
         case TRADE_RETCODE_DONE_PARTIAL:
           {
            Done=true;
           }
         break;

         //---Invalid stops
         case TRADE_RETCODE_INVALID_STOPS:
           {
            m_ResultMsg=RequestErrStr(0);
            Done=false;
           }
         break;

         //---Requote
         case TRADE_RETCODE_REQUOTE:
           {
            m_ResultMsg=RequestErrStr(1);
            m_Bid=Trade_result.bid;
            m_Ask=Trade_result.ask;
            ContinueRequest=true;            // to repeat the request
            Done=false;
           }
         break;

         //---Price changed
         case TRADE_RETCODE_PRICE_CHANGED:
           {
            m_ResultMsg=RequestErrStr(2);
            m_Bid=Trade_result.bid;
            m_Ask=Trade_result.ask;
            ContinueRequest=true;            // to repeat the request
            Done=false;
           }
         break;

         //---No quotes
         case TRADE_RETCODE_PRICE_OFF:
           {
            m_ResultMsg=RequestErrStr(3);
            Done=false;
           }
         break;

         //---many requests
         case TRADE_RETCODE_TOO_MANY_REQUESTS:
           {
            m_ResultMsg=RequestErrStr(4);
            Done=false;
           }
         break;

         //--Market closed
         case TRADE_RETCODE_MARKET_CLOSED:
           {
            m_ResultMsg=RequestErrStr(5);
            Done=false;
           }
         break;

         //--Invalid request
         case TRADE_RETCODE_INVALID:
           {
            m_ResultMsg=RequestErrStr(6);
            Done=false;
           }
         break;

         //--Request rejected
         case TRADE_RETCODE_REJECT:
           {
            m_ResultMsg=RequestErrStr(10);
            m_Bid=Trade_result.bid;
            m_Ask=Trade_result.ask;
            ContinueRequest=true;            // to repeat the request
            Done=false;
           }
         break;

         //---
         default: Print("Retcode ",Trade_result.retcode); break;
        }
      if(m_PlaySound) PlaySound((Done)?OPERATION_SUCCEEDED:OPERATION_FAILED);
     }
   else
     {
      switch(Check_result.retcode)
        {
         case TRADE_RETCODE_INVALID_VOLUME:
           {
            m_ResultMsg=RequestErrStr(7);
            Done=false;
            if(m_PlaySound) PlaySound(OPERATION_FAILED);
           }
         break;

         case TRADE_RETCODE_INVALID_PRICE:
           {
            m_ResultMsg=RequestErrStr(8);
            Done=false;
            if(m_PlaySound) PlaySound(OPERATION_FAILED);
           }
         break;

         case TRADE_RETCODE_NO_MONEY:
           {
            m_ResultMsg=RequestErrStr(9);
            Done=false;
            if(m_PlaySound) PlaySound(OPERATION_FAILED);
           }
         break;

         default: break;
        }
     }
   if(Done) m_ResultMsg=" ";
   return(Done);
  }
//+------------------------------------------------------------------+
