//+------------------------------------------------------------------+
//|                                                 OpenPosition.mq5 |
//|                                              Copyright 2015, Kos |
//|                              https://login.mql5.com/ru/users/kos |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Kos"
#property link      "https://login.mql5.com/ru/users/kos"
#property version   "1.00"
#property description "This script demonstrate work function for opening position"
#property script_show_inputs
//---
#include <Trade\SymbolInfo.mqh>
#include <TM\TM_Lite.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum ENUM_TYPE_ORDER
  {
   BUY                          =0,  //Buy
   SELL                         =1,  //Sell
  };
//--- input parameters
input ENUM_TYPE_ORDER    OrderType=0;
input double   Lot       =0.1;
input int      SL        =0;  //SL in points            
input int      TP        =0;  //TP in points

uchar          m_digits  =0;
double         m_point   =0.0;
double         m_Ask     =0.0;
double         m_Bid     =0.0;
ulong          m_ExpID   =12345; // magic number
ulong          m_Dev     =100;   // deviation
string         m_Comment ="Testing script"; //comment
//---
CSymbolInfo *Info=NULL;
CTradingManager *TM=NULL;
ENUM_ORDER_TYPE Order_Type=ORDER_TYPE_SELL;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool Init(void)
  {
//---create & initilize Info object
   if((Info=new CSymbolInfo)==NULL) return(false);
   else
     {
      Info.Name(Symbol());
      m_digits           =(uchar)Info.Digits();
      m_point            =(double)Info.Point();
      if(!Info.RefreshRates()) return(false);
      m_Ask=(double)Info.Ask();
      m_Bid=(double)Info.Bid();
     }

//---create & initilize TM object   
   if((TM=new CTradingManager)==NULL) return(false);
   else
     {
      if(TM.Initialize(Info)==INIT_FAILED)
        {
         Print("TradingManager initialize failed");
         return(false);
        }
       TM.SetOptions(m_ExpID,m_Dev,m_Comment);
     }

   if(OrderType==0 || OrderType==1)
     {
      Order_Type=(OrderType==0)?ORDER_TYPE_BUY:ORDER_TYPE_SELL;
     }
   return(true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void DeInit(void)
  {
   if(Info!=NULL) delete Info; // destroy Info object
   if(TM!=NULL) delete TM; //destroy TM object
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Processing()
  {
   double _OpenPrice=(Order_Type==ORDER_TYPE_BUY)?(double)m_Ask:(double)m_Bid;
   double _SLPrice=0.0;
   double _TPPrice=0.0;
   bool   _Result=false;
  
   if(SL!=0 && SL>0) _SLPrice=(Order_Type==ORDER_TYPE_BUY)?(_OpenPrice-NormalizeDouble(SL*m_point,m_digits)):(_OpenPrice+NormalizeDouble(SL*m_point,m_digits));

   if(TP!=0 && TP>0) _TPPrice=(Order_Type==ORDER_TYPE_BUY)?(_OpenPrice+NormalizeDouble(TP*m_point,m_digits)):(_OpenPrice-NormalizeDouble(TP*m_point,m_digits));

   if(Lot!=0)
     {
      if(Lot<=Info.LotsMax() && Lot>=Info.LotsMin())
       {
        _Result=TM.OpenPosition(Order_Type,Lot,_SLPrice,_TPPrice);
        if(!_Result)Print(TM.ResultMsg());
       }
       else Print("Invalid volume");
     }
  }
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
   if(Init())
     {
      Processing();
      DeInit();
     }
  }
//+------------------------------------------------------------------+
